use itertools::join;
use std::{cell::RefCell, rc::Rc};
use yew::Callback;

use crate::{
    echo_to_term_frame, lua_state::LuaState, parsing::parse_commands, RegisteredTermFrameLens,
    TermFrame,
};

fn reentrant_command_handler(
    lua_state: &mut LuaState,
    frames: &RegisteredTermFrameLens,
    term_frame: &TermFrame,
    command_in: &str,
) {
    web_sys::console::log_1(&"Inside command handler".into());
    echo_to_term_frame(frames, term_frame, "Hello World!\n");
    for command in parse_commands(command_in).commands {
        match command.split_out_command() {
            None => (),
            Some((cmd, rest)) => {
                if cmd == "##" {
                    match lua_state.execute(&join(rest.arguments.iter(), " ")) {
                        Ok(msg) => echo_to_term_frame(frames, term_frame, &msg).unwrap_or(()),
                        Err(msg) => echo_to_term_frame(frames, term_frame, &msg).unwrap_or(()),
                    }
                }
            }
        }
    }
}

fn command_handler(
    lua_state: &RefCell<LuaState>,
    frames: &RegisteredTermFrameLens,
    term_frame: &TermFrame,
    command_in: String,
) {
    match lua_state.try_borrow_mut() {
        Err(_) => echo_to_term_frame(
            frames,
            term_frame,
            "Attempt to re-enter command handler during processing.\n",
        )
        .unwrap_or(()), // Ignore error handling error.
        Ok(mut lua_state_m) => {
            reentrant_command_handler(&mut lua_state_m, frames, term_frame, &command_in)
        }
    }
}

pub fn make_command_handler_callback(
    lua_state: Rc<RefCell<LuaState>>,
    frames: RegisteredTermFrameLens,
) -> Callback<(TermFrame, String), ()> {
    Callback::from(move |(term, cmd)| command_handler(&lua_state, &frames, &term, cmd))
}
