use std::cell::RefCell;
use std::rc::Rc;

use yew::prelude::*;

pub mod command_handler;
pub mod lineengine;
pub mod lua_state;
pub mod parsing;
pub mod term_view;
use crate::command_handler::*;
use crate::lua_state::LuaState;
use crate::term_view::*;

#[derive(Properties)]
struct GlobalState {
    frame_registry: RegisteredTermFrames,
    lua_engine: LuaState,
}

// Used only for yew. Always equal since we don't want it to
// actually look into GlobalState, changes there should never
// cause a re-render.
impl PartialEq for GlobalState {
    fn eq(&self, _other: &Self) -> bool {
        true
    }
}

#[function_component(App)]
fn app() -> Html {
    let global = use_mut_ref(|| Global {
        frame_registry: RegisteredTermFrames::new().into(),
        lua_engine: LuaState::setup(frames.clone()).expect("Can create interpreter"),
    });

    html! {
        <div class="vpane toplevel">
            <TermView terminal={TermFrame(0)} frames={frames.clone()} handler={command_handler.clone()}/>
            <TermView terminal={TermFrame(1)} frames={frames.clone()} handler={command_handler.clone()}/>
        </div>
    }
}

fn main() {
    console_error_panic_hook::set_once();

    yew::Renderer::<App>::new().render();
}
